# -*- coding: utf-8 -*-
import ExasolDatabaseConnector.ExaWebSockets
from ExasolDatabaseConnector.ExaOdbcDriver.FindDriver   import GetDriverName

driverName = GetDriverName()

if driverName and driverName != '':
    class Database(ExasolDatabaseConnector.ExaOdbcDriver.Database):
        def __init__(self, connectionString, user, password, autocommit = False):
            super().__init__(connectionString, user, password, autocommit)

else:
    class Database(ExasolDatabaseConnector.ExaWebSockets.Database):
        def __init__(self, connectionString, user, password, autocommit = False):
            super().__init__(connectionString, user, password, autocommit)





# -*- coding: utf-8 -*-
from configparser   import ConfigParser
from os             import sep
from os.path        import expanduser, isfile, isdir
from glob           import glob
from platform       import uname

def GetDriverNameFromConfig(odbcIniFile):
    config = ConfigParser()
    config.read(odbcIniFile)
    result = None

    if 'ODBC Data Sources' in config.sections():
        driverName = None
        section = config['ODBC Data Sources']
        for key in section.keys():
            if str(key).startswith('exasolution-'):
                driverName = key
                break
        if driverName and driverName in section and isfile(config[driverName]['DRIVER']):
            result = config[driverName]['DRIVER']
    return result

def GetDriverType():
    files = glob('/usr/lib/*/libodbc.so.*.0.0') + \
            glob('/usr/lib/*/libodbcinst.so.*.0.0') + \
            glob('/usr/lib/*/libodbc.so')

    for item in files:
        if item.endswith('2.0.0'):  return 'uo2214lv2'
        if item.endswith('1.0.0'):  return 'uo2214lv1'
        if item.endswith('.so'):    return 'dd'
    return None

def GetDriverName():
    driverType = GetDriverType()
    if not driverType: 
        return None

    result = None
    odbcIniFile = expanduser("~") + sep + '.odbc.ini'
    if isfile(odbcIniFile):
        result = GetDriverNameFromConfig(odbcIniFile)

    odbcIniFile = '/etc/odbc.ini' 
    if not result and isfile(odbcIniFile):
        result = GetDriverNameFromConfig(odbcIniFile)

    if not result and isdir('/opt/exasol'):
        for item in glob('/opt/exasol/EXASOL_ODBC-*'):
            platformDir = item + sep + 'lib' + sep + str(uname()[0]).lower() + sep + str(uname()[4]).lower()
            if isdir(platformDir):
                driver = platformDir + sep + 'libexaodbc-' + driverType + '.so'
                if isfile(driver):
                    return driver
    return result



# -*- coding: utf-8 -*-
import re
import socket
import ssl
from random import shuffle
from contextlib import closing

class DatabaseAbstract:
    _buffer = None
    _method = None
    __ip4RangePattern = re.compile(r'^(\d+\.\d+\.\d+)\.(\d+)\.\.(\d+)')


    def __init__(self, connectionString, user, password, autocommit = False):
        self._buffer = [] #initialize buffer, don't forget in your inherited method!
        self._method = 'ABSTRACT'
        raise NotImplementedError()


    def getMethod(self):
        return self._method


    def ipFromConnectionString(self, connectionString):
        """chooses an usable IP address from the connection string
            
            Note:
                This method chooses an IP address from the connection string and checks if 
                the port on the given address is open. If the port is closed the next address
                will be checked.

            Args:
                connectionString:   an Exasol database connection string

            Returns:
                Tuple (ip, port) with valid address or None if no ip is usable

        """
        connectionSplit = connectionString.split(':')
        ipString = connectionSplit[0]
        port = int(connectionSplit[1])

        ipItems = []
        for ipRange in ipString.split(','):
            if not '..' in ipRange: #not a range, just an single IP
                ipItems.append(ipRange)
            else:
                match = self.__ip4RangePattern.match(ipRange)
                for i in range(int(match.group(2)), int(match.group(3)) + 1):
                    ipItems.append('%s.%i' % (match.group(1), i))
        shuffle(ipItems)
        for ip in ipItems:
            with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
                if s.connect_ex((ip, port)) == 0:
                    return (ip, port)
        return None


    def escapeIdent(self, text):
        """Escapes SQL identfiers

            Args:
                text:       SQL identifier that needs to be escaped

            Returns:
                The provided identifier, escaped
        """
        return '"' + text.replace('"', '""') + '"'


    def escapeString(self, text):
        """Escapes SQL strings

            Args:
                text:       SQL string that needs to be escaped

            Returns:
                The provided string, escaped
        """
        return "'" + text.replace("'", "''") + "'"


    def getBuffer(self):
        """Creates a string containing the current SQL buffer
            
            Args:
                None

            Returns:
                A string containing all SQL commands on the SQL buffer
        """
        return '\n'.join(self._buffer)


    def cleanBuffer(self):
        """Purges the SQL buffer content
            
            Args:
                None

            Returns:
                None
        """
        self._buffer = [] 


    def addToBuffer(self, sqlText):
        """Appends a SQL statement to the internal SQL buffer

            Args:
                sqlText:    The statement which will be appended

            Returns:
                None
        """
        self._buffer.append(sqlText.strip())


    def close(self):
        raise NotImplementedError()

    def execute(self, sqlText, *args):
        raise NotImplementedError()

    def executeBuffer(self):
        raise NotImplementedError()



# -*- coding: utf-8 -*-
import ssl
from ExasolDatabaseConnector.ExaDatabaseAbstract    import DatabaseAbstract
from EXASOL                                         import connect, cursor

class Database(DatabaseAbstract):
    """The Database class easifies the access to your DB instance

    Using the Database class is quite easy:

        #create a new instance
        db = Database(username, password, autocommit = False)

        #do a single query (prepared statement)
        result = db.execute('SELECT USER_NAME FROM SYS.EXA_DBA_USERS WHERE USER_NAME = ?;', username)
        for row in result:
            pprint(row)

        #execute multiple SQL statements:
        db.addToBuffer('CREATE SCHEMA "test123";')
        db.addToBuffer('DROM SCHEMA "test_old";')
        db.executeBuffer()
    
        #close the connection
        db.close()

        Args:
            connectionString (str): Exasol database connection string
            user (str):         username used to login into DB instance
            password (str):     password used to login into DB instance
            autocommit (bool, optional): enables or disables autocommit
    """

    __conn = None
    __connectionTuple = None

    def __init__(self, connectionString, user, password, autocommit = False):
        self._method = 'WebSockets'
        self.__connectionTuple = self.ipFromConnectionString(connectionString)
        if self.__connectionTuple:
            self.__conn = connect(
                'wss://%s:%s' % self.__connectionTuple,
                user,
                password,
                autocommit,
                sslopt={"cert_reqs": ssl.CERT_NONE}
            )
            self._buffer = []


    def execute(self, sqlText, *args):
        """Executes a single SQL statement

            Note:
                This method is to execute a single SQL statement and retrieving the result. If you try
                to execute more than one statement use .addToBuffer() and .executeBuffer() instead.

            Args:
                sqlText:    The SQL statement which should be executed on the DB instance
                *args:      All variables which are necessary to execute a prepared statement; 

            Returns:
                None:       If no result is present
                List:       A list of all result rows
        """
        result = []
        with self.__conn.cursor() as c:
            if c.execute(sqlText, *args) > 0:
                for row in c:
                    result.append(row)
            return result
        return None


    def close(self):
        """Closes the connection to the database instance
            
            Args:
                None

            Returns:
                None
        """
        self.__conn.close()


    def executeBuffer(self):
        """Executes the content of the internal SQL buffer on the DB instance

            Args:
                None

            Return:
                A list with all results for each line
        """
        results = []
        for sql in self._buffer:
            results.append(self.execute(sql))
        self.cleanBuffer()
        return results




#!/usr/bin/env python
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='ExasolDatabaseConnector',
    version="0.0.8",
    license="MIT",
    maintainer="Florian Reck",
    maintainer_email="support@exasol.com",
    description="Exasol database connector class written in python",
    long_description="Exasol database connector classes using ODBC or WebSockets",
    url='https://github.com/florian-reck/ExaDatabase',
    packages=[
        'ExasolDatabaseConnector',
        'ExasolDatabaseConnector.ExaDatabaseAbstract',
        'ExasolDatabaseConnector.ExaWebSockets',
        'ExasolDatabaseConnector.ExaOdbcDriver'
    ],
    install_requires=[
        'websocket_client',
        'rsa',
	'EXASOL-DB-API',
        'pyodbc'
    ]
)
